@@ -948,9 +948,8 @@ int setup_files(struct thread_data *td)
 	if (need_extend) {
 		temp_stall_ts = 1;
 		if (output_format & FIO_OUTPUT_NORMAL)
-			log_info("%s: Laying out IO file(s) (%u file(s) /"
-				 " %lluMB)\n", o->name, need_extend,
-					extend_size >> 20);
+			log_info("%s: Laying out IO file(s) (%u file(s) / %lluMiB)\n",
+				 o->name, need_extend, extend_size >> 20);
 
 		for_each_file(td, f, i) {
 			unsigned long long old_len = -1ULL, extend_len = -1ULL;
@@ -1098,7 +1097,7 @@ static int check_rand_gen_limits(struct thread_data *td, struct fio_file *f,
 	if (!fio_option_is_set(&td->o, random_generator)) {
 		log_info("fio: Switching to tausworthe64. Use the "
 			 "random_generator= option to get rid of this "
-			 " warning.\n");
+			 "warning.\n");
 		td->o.random_generator = FIO_RAND_GEN_TAUSWORTHE64;
 		return 0;
 	}
